##############################################################################
# Description
#     This is the virtual controller of the machine tool driver for 
#     a generic 5-axis head-head configuration. 
#     The driver is defined by configuring a generic drive
#
# Revisions
#
#   Date        Who   Reason
# 01-Jan-2002   yxd	  First Version
# 28-Feb-2002   yxd	  Added  hole cycles simulation
# 15-May-2002   yxd	  Changed name of advance kinematics dll to ugpadvkins.dll
# 25-Jun-2002   yxd   Added setting rotary axis direction
# 30-Jul-2002   yxd   Define by configuring a generic driver
# 28-Nov-2002   Wilson Kurniawan Changed default system dialog table Syntax  
# 11-Jun-2003   JM             little fixes
#
#
# $HISTORY$
#
###############################################################################


#______________________________________________________________________________
# The following command invokes the corresponding event handler to generate the
# NC program
#______________________________________________________________________________

set the_cam_post_dir  [MOM_ask_env_var UGII_CAM_POST_DIR]
set the_cam_debug_dir [MOM_ask_env_var UGII_CAM_DEBUG_DIR]


# Simulation generic virtual controller
source ${the_cam_post_dir}/dmu_tt_sim_vnc_cmds.tcl

#______________________________________________________________________________
# The following command invokes the debugging mode.
#______________________________________________________________________________
#  source ${the_cam_debug_dir}/mom_debug.tcl
#  source ${the_cam_debug_dir}/mom_review_mtd.tcl
#********Setting the debug mode ON/OFF
   MOM_set_debug_mode OFF


#
# the following function configures the simulation driver
# to function as a 5-axis driver for a 5-axis machine
# this function is called by the generic driver during 
# initialization
#

proc VNC_configure_driver { } {

  global sim_cfg_kin_machine_type
  global sim_cfg_4th_axis_name
  global sim_cfg_4th_axis_name
  global sim_cfg_5th_axis_name
  global sim_cfg_driver_name
  global sim_cfg_linear_axes_map 
  global sim_cfg_rotary_axes_map 
  global sim_cfg_linear_axes_config  
  global sim_cfg_rotary_axes_config 
  global sim_cfg_4th_axis_rotary_dir_mode 
  global sim_cfg_5th_axis_rotary_dir_mode 
  global sim_cfg_4th_axis_allow_reversal
  global sim_cfg_5th_axis_allow_reversal 
  global sim_cfg_debug
  global sim_cfg_tool_mounting_component
  global sim_cfg_tool_mounting_junction
  global sim_cfg_tool_mounting_time
  global sim_cfg_tool_unmounting_time
  global sim_cfg_machine_tool_ref_junction
  global sim_cfg_setup_post_kinematics_sw
  global sim_cfg_setup_post_kinematics_ref_junction 
  global sim_cfg_duration_callback_fct
  global sim_cfg_tool_change_motion_callback_fct
  global sim_cfg_tool_ref_junction
  global sim_cfg_end_of_prog_callback_fct
  global sim_cfg_mounting_zcs_component
  global sim_cfg_setup_advance_kinematics_sw

# (REQUIRED)
#  the logical names of the 4'th and 5'th axis. 
#  must be set for 4 or 5 axes machine
  set sim_cfg_4th_axis_name C
  set sim_cfg_5th_axis_name B

#(REQUIRED)
# define mapping between logical and phisical axes as they named
# on the machine. This has to be defined for the linear and rotary axes.
  set sim_cfg_rotary_axes_map [concat C C B B]
  set sim_cfg_linear_axes_map [concat X X Y Y Z Z]

# (REQUIRED)
# define the linear and rotary axes that participate in execution of the
# motion. Use the logical names of the axes.
# For example, for this machine all axes participate in executing a 
# certain motion. so all axes are included in axes configuration.  
  set sim_cfg_rotary_axes_config [concat C B]
  set sim_cfg_linear_axes_config [concat X Y Z]

#(REQUIRED)
# define on which K-component the tool is mounted and to which junction
# it is attached.
  set sim_cfg_tool_mounting_component "SPINDLE"
  set sim_cfg_tool_mounting_junction  "SPINDLE@TOOL_MOUNT_JCT"

#(REQUIRED)
# when the tool is not mounted on the machine a new reference
# junction has to be defined to replace the tool tip junction. 
# this reference junction is used to move the machine without a tool.
 set sim_cfg_machine_tool_ref_junction "SPINDLE@TOOL_MOUNT_JCT"

# (REQUIRED)
# specify on which K-component to mount the ZCS junction. This
# is the junction that matchs the MSYS coordinate system. The 
# NC program coordinates are i.r.t this coordinate system
# Note: don't position ZCS on a K-component that moves or rotate 
# since ZCS most oftime has to be stationary. Except for example 
# a table mounted on linear axis where the control expect the 
# coordinates in the moving ZCS coordinate system. In that case
# the ZCS is mounted on the moving linear axis
 set sim_cfg_mounting_zcs_component "MACHINE_BASE"

# (OPTIONAL)
# name of driver
  set sim_cfg_driver_name "DECKEL-MAHO 5 EJES EN MESA A 45 GRADOS" 

# (OPTIONAL)
# set rotation direction mode which specify in which direction to rotate
# the rotary axis when moving to target position. by default shortest 
# path is applied (if not specified)
#  set sim_cfg_4th_axis_rotary_dir_mode "ALWAYS_SHORTEST"
#  set sim_cfg_5th_axis_rotary_dir_mode "ALWAYS_SHORTEST"

# (OPTIONAL)
# set whether axis reversal is allowed. i.e. when axis cannot reach target
# position through the shortest path whether going on the longest path is
# allowed. For example if current position is 350 and target is 10 is limits
# don't allow going to 10. this controls whether it can rotate 240 degrees to 
# 10. By default it is allowed for all rotary axes.
#  set sim_cfg_4th_axis_allow_reversal "YES"
#  set sim_cfg_5th_axis_allow_reversal "YES"

# (OPTIONAL)
# set the callback TCL procedure for computing the duration time. If not
# set a default one is provided.
# The function has the following arguments
#   CalculateDurationTime { linear_or_angular delta } where
#      linear_or_angular is either "LINEAR" or "ANGULAR" which defines
#      whether the incremental displacement (delta) is linear(in current units) 
#      or angular(in degrees).
#
#   The function must return the duration in seconds to perform this delta
#   angular or linear motion.
#
# set sim_cfg_duration_callback_fct "SIM_calculate_duration_time_CB"

# (OPTIONAL)
# This defines the function that is called to perform the motion to the 
# tool change position. Note that this is just the motion not the actual tool change.
# if not set, tool change is done in current position. 
# The function gets no arguments and return nothing. It is just performs the
# motion.
#
# the function  is defined later in this file
 set sim_cfg_tool_change_motion_callback_fct "SIM_tool_change_path_CB"

# (OPTIONAL)
# This function is called when end of program is triggered. 
# The function gets no arguments and returns nothing. 
# if not defined. no motion is performed.
#
  set sim_cfg_end_of_prog_callback_fct "SIM_end_of_program_CB"

# (OPTIONAL)
# set debug options  
# Set what outputs we want to see. Valid tokens are:
# SIM_DBG_OUTPUT_ERROR, SIM_DBG_OUTPUT_NC_CMD, SIM_DBG_OUTPUT_CEVENT, 
# SIM_DBG_OUTPUT_INPUT_SV_EVENT, SIM_DBG_OUTPUT_OUTPUT_SV_EVENT, 
# SIM_DBG_OUTPUT_PROCESSED_SV_EVENT, SIM_DBG_OUTPUT_SV_CMD, SIM_DBG_OUTPUT_ALL
# 
#  set sim_cfg_debug "SIM_DBG_OUTPUT_INPUT_SV_EVENT SIM_DBG_OUTPUT_CEVENT"
#
# when debug is set you can use the following command to write text
# SIM_dbg_write_message ">>> This is a testing line <<<<"
#

#(OPTIONAL)
# the time it takes to mount and unmount a tool in seconds. if not define mount and unmount
# are set as half of the mom_kin_tool_change_time value if defined.
 set sim_cfg_tool_mounting_time   6
 set sim_cfg_tool_unmounting_time 6
 
# (OPTIONAL)
# When the machine is instructed to move, by default (if variable not set) this means
# to bring the tool tip position to target position. Any other junction can be 
# defined as the reference junction. Make sure the junction is attached to 
# the right K-component.
# set sim_cfg_tool_ref_junction "SPINDLE@SPINDLE_JCT"
     
#(OPTIONAL)
# The following switch control whether to setup the post kinematics
# based on the machine configuration. If not set, the pos kinematics 
# of the machine has to be defined outside VNC.
# The kinematics parameters that are set are defined in sim_mtd_kinematics.tcl.
# Those parameters are just the kinematics geometry like axes limits, rotation
# centers and others 
# Variable values:0 or 1
 set sim_cfg_setup_post_kinematics_sw 1

#(OPTIONAL)
# if sim_cfg_setup_post_kinematics_sw is set then...
# define the reference junction for defining the 4th and 5th
# axis post kinematics data. It can be the current ZCS junction or 
# an explicit one. Offsets and vectors for POST kinematics are defined
# i.r.t this junction. If you use ZCS, make sure that the right 
# ZCS is active when this procedure is called.
# To use the current ZCS set the following to "_SIM_CURRENT_ZCS_JUNCTION"
# otherwise use an explicit junction.
#  set sim_cfg_setup_post_kinematics_ref_junction "ROT_JCT"
set sim_cfg_setup_post_kinematics_ref_junction "_SIM_CURRENT_ZCS_JUNCTION"

# (OPTIONAL)
# specify whether to setup the advance kinematics DLL too. this will
# load the advance kinematics DLL and reload its kinematics
# by default it is not loaded (set to 0)
# Variale values: 0 or 1
# set sim_cfg_setup_advance_kinematics_sw 1

#
# use the following commands to customize the simulation dialog. See Custom
# Dialog Spec for command parameters.
#

# SIM_dialog_start "M5ACHH Generic Driver Controller"

# system items
# SIM_dialog_add_item    "SYS_ACTIVE_TOOL_ITEM     S1"
# SIM_dialog_add_item    "SYS_SPEED_ITEM           S2" 
# SIM_dialog_add_item    "SYS_FEED_ITEM            S3 ATTACHMENT=<LEFT 0 82>"
# SIM_dialog_add_item    "SYS_MACHINE_TIME_ITEM    S4"
# SIM_dialog_add_item    "SYS_COOLANT_ITEM         S5 ATTACHMENT=<LEFT 0 30>"
# SIM_dialog_add_item    "SYS_POSITION_ITEM        S6"
# SIM_dialog_add_item    "SYS_PROGRAM_WINDOW_ITEM  S7"
# SIM_dialog_add_item    "SYS_MESSAGE_WINDOW_ITEM  S8"
# SIM_dialog_add_item    "SYS_SHOW_IPW_ITEM        S9"
# SIM_dialog_add_item    "SYS_STEP_CONTROL_ITEM    S10"
# SIM_dialog_add_item    "SYS_ANIMATION_SPEED_ITEM S11"

# user items
# SIM_dialog_add_item     "USR_SEPARATOR_ITEM      U1"
# SIM_dialog_add_item     "USR_LABEL_ITEM          U2 LABEL=<my label>"
# SIM_dialog_add_item     "USR_STRING_ITEM         U3 LABEL=<my string> VALUE=<string value>"
# SIM_dialog_add_item     "USR_RADIO_ITEM          U4 LABEL=<my radio> ORIENTATION=<HORIZONTAL> SPAN=<3> LIST=<radio1,radio2,radio3> VALUE=<1> FORMAT=<CHECK_BOX>"

# SIM_dialog_end 

#
# Customized items can be changed while running machine tool driver. Those
# commands can be executed after MTD has been initialized
#
# For example for the position item to show just the linear axis X, Y and Z
#    SIM_dialog_set_item "S6 LIST=<X,Y,Z>"
# 
# or for example to dim the collant item do
#    SIM_dialog_set_item "S5 SENSITIVE=<OFF>" 
#
# For user item all attributes can change
#   SIM_dialog_set_item     "U2 LABEL=<new label> SENSITIVE=<OFF>"
#   SIM_dialog_set_item     "U3 LABEL=<new label> VALUE=<new value> SENSITIVE=<OFF>"
#   SIM_dialog_set_item     "U4 LABEL=<new label> VALUE=<2> SENSITIVE=<OFF>"
#
}

#
# the callback that describes the path to the tool change position.
# this is called when the tool is changed
#
proc SIM_tool_change_path_CB { } {

   #
   # this moves the machine tool reference junction (sim_cfg_machine_tool_ref_junction)
   # to the specified position. its recommented to specify the motion i.r.t
   # the MTCS coordinate system
   #
   VNC_move_linear_mtcs RAPID Z -2
}

#
# this call back is called when end of program event is triggered
#
proc SIM_end_of_program_CB { } {

 global sim_4th_axis sim_5th_axis
 global mom_sys_home_pos

 # this sets the reference junction to be the pivot point
 # of the machine tool this to allow to specify coordintaes i.r.t
 # machine tool reference junction (sim_cfg_machine_tool_ref_junction)
 VNC_set_ref_jct "undefined"

 VNC_move_linear_mtcs RAPID Z $mom_sys_home_pos(2)
 VNC_move_linear_mtcs RAPID X $mom_sys_home_pos(0) Y $mom_sys_home_pos(1) $sim_4th_axis 0 $sim_5th_axis 0
}
