##############################################################################
# Description
#     This is the configurable virtual controller for generic 5-axis machine.
#     In order to setup 3 to 5 axes VNC driver, the global variables that are
#     listed in the procedure VNC_configure_driver_sample must be set. 
#     Detailed explanation is included in that procedure that is enclosed at the
#     end of this file.
#     The function VNC_configure_driver has to be defined by the user of this
#     generic driver. During initialization of the driver this function
#     will be called.
#     If you have a machine tool with a different configuration than use this
#     VNC as a basis for defining the VNC driver.
#
# Revisions
#
#   Date        Who            Reason
# 17-Jul-2002   yxd				First Version
# 26-Nov-2002   Thomas Schulz Add VNC_nurbs_move
# 28-Nov-2002   Wilson Kurniawan Changed default system dialog table Syntax
# 04-Jun-2003   Jeremy Wight     PR4784257 support SFM and SMM speeds
# 11-Jun-2003   JM             little fixes
#
#
# $HISTORY$
#
###############################################################################


#______________________________________________________________________________
# The following command invokes the corresponding event handler to generate the 
# NC program
#______________________________________________________________________________

set the_cam_post_dir  [MOM_ask_env_var UGII_CAM_POST_DIR]
set the_cam_debug_dir [MOM_ask_env_var UGII_CAM_DEBUG_DIR]


# Simulation include files
  source ${the_cam_post_dir}/sim_high_level_sv_commands.tcl

  
#______________________________________________________________________________
# The following command invokes the debugging mode.
#______________________________________________________________________________
#  source ${the_cam_debug_dir}/mom_debug.tcl
#  source ${the_cam_debug_dir}/mom_review_mtd.tcl
#********Setting the debug mode ON/OFF
#   MOM_set_debug_mode OFF


#
# ====== define some local parameters =========
#
  set sim_prev_tool_name         "undefined"
  set sim_prev_zcs               "undefined"
  set sim_feedrate_mode          "RAPID"
  set sim_advance_kin_dll_loaded 0
  set sim_mtd_initialized        0

#===== set global UG variables that are used here in generic VNC
  set feed  0
  set mom_feed_rate_mode "UNKNOWN FEEDRATE UNITS" 
#****************************************************

#
#========== SET globals for machine tool driver ==============
#
#
#
# As part of the initialization of the machine tool driver, 
# the MOM_SIM_initialize_mtd event is triggered and the MOM_SIM_initialize_mtd 
# procedure is called. First thing that this procedure does it invokes 
# the S&V command to initialize the machine tool driver and to set the virtual 
# controller mode. The S&V command is SIM_mtd_init and it accepts one argument, 
# the controller mode. Currently there are three virtual controller modes:
# 
#    NC_CONTROLLER_SIM_EVENT_HANDLER: A dedicated SIM Event is triggered for each Post
#                                  event. The MTD file (i.e. the TCL file) should
#                                  contains TCL procedures to handle those SIM events.
#                                  For example MOM_SIM_linear_move simulates the linear 
#                                  motion event. Those SIM event handlers should call
#                                  the corresponding S&V commands to move axis, mount 
#                                  tools and devices, set parameters and issue different 
#                                  feedback 
#
#    NC_CONTROLLER_PLUGIN: At each event (when applicable)  MOM POST sets the TCL
#                       global variable MOM_SIM_NC_COMMAND with the next NC program line
#                       to execute. Then the event MOM_SIM_execute_nc_command 
#                       is triggered to process the NC program command stored in variable.
#
#    NC_CONTROLLER_MTD_EVENT_HANDLER: The simulation handling is mixed with NC-program
#                                  handling. No Special handling for MTD is defined.
#                                  S&V commands can be mixed with other post commands.
#
# When MTD ends its execution the event MOM_SIM_exit_mtd is triggered to do the proper 
# cleanup of the machine tool driver.
#
# This files contain an example for setting the machine tool driver to work
# in NC_CONTROLLER_SIM_EVENT_HANDLER. Look at the mill3x_mtd_plugin.tcl for 
# an example on how to set the MTD to work in NC_CONTROLLER_PLUGIN mode.
#
#
#_______________________________________________________________________________
proc MOM_SIM_initialize_mtd { } {
#_______________________________________________________________________________
# This procedure is executed before the simulation starts. it suppose to initialize 
# the machine tool driver.
#_______________________________________________________________________________
#
   global sim_mtd_initialized


#---- initializing MTD and setting controller mode. see description of modes above
   SIM_mtd_init NC_CONTROLLER_MTD_EVENT_HANDLER

#
#---- this function configures the generic driver and is defined by
#     the external user of the generic driver. A sample of this function
#     is appended to this file as VNC_configure_driver_sample function
#   
   VNC_configure_driver
#
#---- check that all configuable VNC variable are set properly
#
   vnc_init_config


  #set default reference junction
  VNC_set_ref_jct "undefined"   

# for internal use flag to check whether the post is running in simulation
# or for NC files creation.
  set sim_mtd_initialized 1
}

#_______________________________________________________________________________
proc MOM_SIM_exit_mtd { } {
#_______________________________________________________________________________
# This procedure exits the machine tool driver. this procedure 
# is called when the machine tool simulation ends.
#_______________________________________________________________________________
#
  global sim_cfg_debug

# invoke S&V command to exit machine tool driver. 
     SIM_mtd_reset 

  if {[info exists sim_cfg_debug] != 0} {
    SIM_dbg_end 
  }
}

#
# Just in case this handler is not defined, then set its default behavior
#
proc MOM_msys {} {

#
# set the current MSYS as a new ZCS. The NC program is defined in
# the ZCS ccordinate system 
  VNC_set_msys_as_zcs
}


#=============================================================
proc VNC_start_of_program { } {
#=============================================================
#_______________________________________________________________________________
# This procedure is executed at the very begining of the program.
# It gets called before any command is read from the task file.
# here we can do any machine tool initialization 
#_______________________________________________________________________________

  global sound_files_library 
  global mom_output_unit
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  SIM_start_of_simulation

# set the machine tool driver units
  if { $mom_output_unit == "MM" } {
     SIM_set_mtd_units "MM" 
  } else {
     SIM_set_mtd_units "INCH" 
  }

 #
 #init reference junction
 VNC_set_ref_jct "undefined"

}

#=============================================================
proc VNC_end_of_program { } {
#=============================================================

    global sim_mtd_initialized
    global sim_cfg_end_of_prog_callback_fct
#    global sim_4th_axis sim_5th_axis
#    global sim_prev_tool_name

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }
   
    if {[info exists sim_cfg_end_of_prog_callback_fct] != 0} {
        $sim_cfg_end_of_prog_callback_fct
    }

    SIM_end_of_simulation
}


#
# create and set the current ZCS junction
# this function is called when the MOM_msys is triggered
#
#=============================================================
proc VNC_set_msys_as_zcs {  } {
#=============================================================
#
# this event is trigered when the msys is changed
#
 global mom_msys_origin mom_msys_matrix 
 global sim_rotate_table_switch  mom_sim_result 
 global sim_mtd_initialized
 global sim_cfg_mounting_zcs_component

 # check if simulation has been initialized properly
 if { !$sim_mtd_initialized } { return }

 # mount the junction on the rotary table 
 set dest_cmp $sim_cfg_mounting_zcs_component

 #
 # convert part/simulation units to MTD units
 lappend  origin [ SIM_convert_sim_to_mtd_units $mom_msys_origin(0) ]
 lappend  origin [ SIM_convert_sim_to_mtd_units $mom_msys_origin(1) ]
 lappend  origin [ SIM_convert_sim_to_mtd_units $mom_msys_origin(2) ]

 # convert matrix vector to list
 for {set i 0} {$i<9} {incr i}  {
   lappend mtx $mom_msys_matrix($i)
 }

 # check if ZCS junction already created. if yes, return its name. if not create new one
 # and return its name. this optimization is needed because this event is invoked
 # before each operation is exceuted.
 set zcs_name [ SIM_create_zcs_junction CURRENT_ZCS_JCT $dest_cmp $origin  $mtx ]
 
 SIM_update_current_zcs $zcs_name

}

#=============================================================
proc VNC_send_message { message } {
#=============================================================
 global sim_mtd_initialized

 # check if simulation has been initialized properly
 if { !$sim_mtd_initialized } { return }


 SIM_feedback_message "MTD:" "$message"
  
}

#-------------------------------------------------------------------------------
# CalculateDurationTime
#
# return the duration time in seconds to pass the delta length in
# current feedrate
# linear_or_angular is either "LINEAR" or "ANGULAR" which defines
# whether delta is linear(in current units) or angular(in degrees) 
# displacement 
#-------------------------------------------------------------------------------
#=============================================================
proc CalculateDurationTime { linear_or_angular delta } {
#=============================================================

	global mom_feed_rate_mode
	global mom_feed_rate
	global mom_output_unit sim_feedrate_mode
    global mom_kin_rapid_feed_rate
    global mom_feed_rate_dpm
    global feed

    set length [expr {abs($delta)}]
    if {$length < 0.00001 } { return 0.0 } 

    # compute duration for angular motion 
    if { $linear_or_angular == "ANGULAR" } {
       set lfeed  $mom_feed_rate_dpm
    } else {
      if { $sim_feedrate_mode == "RAPID" } {
        if {[info exists mom_kin_rapid_feed_rate] != 0 } {
           set lfeed $mom_kin_rapid_feed_rate 
        } else {
          set lfeed $feed
        }
      } else {
        set lfeed $feed
      }

      if {$mom_feed_rate_mode != "MMPM" } {
		set lfeed [expr {$lfeed * 25.4}] 
      } 

	  if {$mom_output_unit != "MM" } \
		{ set length [expr {$length * 25.4}] }
    }
	
	if { $lfeed < 0.00001 } \
		{ set time 0 } \
	else \
		{ set time [expr {($length / $lfeed) * 60.0}] }

    return $time
}


#=============================================================
proc VNC_circular_move { } {
#=============================================================

  global mom_feed_rate
  global mom_spindle_speed
  global mom_arc_direction
  global mom_pos_arc_center
  global mom_pos_arc_axis
  global mom_pos mom_out_angle_pos
  global xfc yfc xc yc

  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "CUT"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]

  if {$mom_arc_direction == "CLW"} {
     set dir -1.0
  } else {
     set dir 1.0
  }

  set i $mom_pos_arc_center(0) 
  set j $mom_pos_arc_center(1)
  set k $mom_pos_arc_center(2)

  SIM_move_circular_zcs 0 0 $dir X $xfc Y $yfc I $xc J $yc

#  if {[EQ_is_equal abs($mom_pos_arc_axis(2)) 1.0]} {
#     SIM_move_circular_zcs 0 0 $dir X $mom_pos(0) Y $mom_pos(1) I $i J $j
#  } elseif {[EQ_is_equal abs($mom_pos_arc_axis(1)) 1.0]} {
#     SIM_move_circular_zcs 0 $dir 0 Z $mom_pos(2) X $mom_pos(0) K $k I $i
#  } elseif {[EQ_is_equal abs($mom_pos_arc_axis(0)) 1.0]} {
#     SIM_move_circular_zcs $dir 0 0 Y $mom_pos(1) Z $mom_pos(2) J $j K $k
#  }
}

#=============================================================
proc VNC_helix_move { } {
#=============================================================

  global mom_feed_rate
  global mom_spindle_speed
  global mom_arc_direction
  global mom_pos_arc_center
  global mom_pos_arc_axis
  global mom_pos mom_out_angle_pos
  global mom_prev_pos
  global mom_sys_helix_pitch_type

  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "CUT"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]

  if {$mom_arc_direction == "CLW"} {
     set dir -1.0
  } else {
     set dir 1.0
  }

  set i $mom_pos_arc_center(0) 
  set j $mom_pos_arc_center(1)
  set k $mom_pos_arc_center(2)

  if {[EQ_is_equal abs($mom_pos_arc_axis(2)) 1.0]} {
    if {$mom_sys_helix_pitch_type != "none"} {
      set p [expr $mom_pos_arc_center(2)-$mom_prev_pos(2)]
      set pitch "P $p"    
    } else {
      set pitch ""
    }

    SIM_move_circular_zcs 0 0 $dir X $mom_pos(0) Y $mom_pos(1) I $i J $j $pitch
  } elseif {[EQ_is_equal abs($mom_pos_arc_axis(1)) 1.0]} {
    if {$mom_sys_helix_pitch_type != "none"} {
      set p [expr $mom_pos_arc_center(1)-$mom_prev_pos(1)]
      set pitch "P $p"    
    } else {
      set pitch ""
    }

    SIM_move_circular_zcs 0 $dir 0 Z $mom_pos(2) X $mom_pos(0) K $k I $i $pitch
  } elseif {[EQ_is_equal abs($mom_pos_arc_axis(0)) 1.0]} {
    if {$mom_sys_helix_pitch_type != "none"} {
      set p [expr $mom_pos_arc_center(0)-$mom_prev_pos(0)]
      set pitch "P $p"    
    } else {
      set pitch ""
    }

     SIM_move_circular_zcs $dir 0 0 Y $mom_pos(1) Z $mom_pos(2) J $j K $k $pitch
  }
}

proc VNC_ask_feedrate_mode { } {
#_______________________________________________________________________________
# This function returns the feedrate units as expected by S&V commands
#_______________________________________________________________________________
  global   mom_feed_rate_mode
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

#  if {[hiset mom_feed_rate_mode]} ;
  if {[info exists mom_feed_rate_mode] != 0 } {
   switch $mom_feed_rate_mode {
    IPM     { set feed_units "INCH_PER_MIN" }
    MMPM    { set feed_units "MM_PER_MIN"}
    IPR     { set feed_units "INCH_PER_REV" }
    MMPR    { set feed_units "MM_PER_REV" }
#    FRN    { set feed_units "MM_PER_100REV" }
    default { set feed_units "UNKNOWN FEEDRATE UNITS" }
   }
  } else {
    set feed_units "UNKNOWN FEEDRATE UNITS" 
  }

  return $feed_units 

}

proc VNC_ask_speed_mode { } {
#_______________________________________________________________________________
# This function returns the speed units as expected by S&V commands
#_______________________________________________________________________________
  global   mom_spindle_mode
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

#  if {[hiset mom_spindle_mode]};
  if {[info exists mom_spindle_mode] != 0 } {
   switch $mom_spindle_mode {
    RPM { set speed_units "REV_PER_MIN" }
    SFM { set speed_units "SFM" }
    SMM { set speed_units "SMM" }
    default { set speed_units "UNKNOWN SPEED UNITS" }
   }
  } else {
    set speed_units "UNKNOWN SPEED UNITS" 
  }

  return $speed_units
}


#=============================================================
proc VNC_set_feedrate_mode { cutting_mode } {
#=============================================================
  global mom_feed_rate sim_feedrate_mode
  global sim_mtd_initialized
  global mom_kin_rapid_feed_rate
  global feed

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  SIM_set_cutting_mode $cutting_mode

  set sim_feedrate_mode $cutting_mode

  if { $cutting_mode == "RAPID" } {
        if {[info exists mom_kin_rapid_feed_rate] != 0 } {
           set lfeed $mom_kin_rapid_feed_rate 
        } else {
          set lfeed $feed
        }
  } else {
        set lfeed $feed
  }

  SIM_set_feed $lfeed [VNC_ask_feedrate_mode]
}

#=============================================================
proc VNC_unmount_tool { tool_ug_name } {
#=============================================================

    global sim_prev_tool_name  
    global mom_sim_result 
    global mom_kin_tool_change_time   
    global sim_mtd_initialized
    global sim_cfg_tool_change_motion_callback_fct
    global sim_cfg_tool_unmounting_time

    # check if simulation has been initialized properly
    if { !$sim_mtd_initialized } { return }

       #VNC_set_feedrate_mode "RAPID"

       #
       #define the reference junction so the tool change position will
       # be i.r.t machine reference junction
       VNC_set_ref_jct "undefined"

        #
        # move to toolchange position 
        #
        if {[info exists sim_cfg_tool_change_motion_callback_fct] != 0 } {
           $sim_cfg_tool_change_motion_callback_fct
        }

       if {$tool_ug_name  != "undefined" } { 
         SIM_ask_kim_comp_name_by_id "TOOL" $tool_ug_name
         set t_name $mom_sim_result
         
         SIM_unmount_tool $sim_cfg_tool_unmounting_time $t_name  
         SIM_update

         set sim_prev_tool_name "undefined"
       }
}

#=============================================================
proc VNC_tool_change {  } {
#=============================================================
    global mom_tool_name
    global sim_prev_tool_name 
    global sound_files_library
    global mom_tool_z_offset 
    global mom_sim_result   
    global sim_cfg_tool_mounting_component
    global sim_cfg_tool_mounting_junction
    global sim_cfg_tool_mounting_time

    global sim_mtd_initialized

    # check if simulation has been initialized properly
    if { !$sim_mtd_initialized } { return }

       set tool_ug_name $mom_tool_name

       set tool_name " "

       if {$sim_prev_tool_name != $tool_ug_name && $tool_ug_name  != ""} {

         VNC_unmount_tool $sim_prev_tool_name

#
#  Specify where (i.e. on which K-Component) the tool should be mounted 
#  (e.g SPINDLE) and what junction on that component should be used for mounting
#  (e.g, SPINDLE_JCT).
#

         SIM_mount_tool $sim_cfg_tool_mounting_time "UG_NAME" $tool_ug_name $sim_cfg_tool_mounting_component $sim_cfg_tool_mounting_junction
         set tool_name $mom_sim_result 

         if {$tool_name != ""} {
            SIM_activate_tool $tool_name
         }

         SIM_update 

         set sim_prev_tool_name $tool_ug_name
         VNC_move_linear_mtcs RAPID Z 200.0


       }

        #
        # set the reference junction
        VNC_set_ref_jct $tool_name
}


#=============================================================
proc VNC_set_ref_jct { tool_name } {
#=============================================================
global sim_mtd_initialized
global sim_cfg_machine_tool_ref_junction
global sim_cfg_tool_ref_junction

# check if simulation has been initialized properly
if { !$sim_mtd_initialized } { return }

         if {$tool_name != " " && $tool_name != "" && $tool_name != "undefined"} \
         {
              if {[info exists sim_cfg_tool_ref_junction] == 0 } {
                set ref_jct $tool_name@SIM_TOOL_TIP
              } else {
                set ref_jct $sim_cfg_tool_ref_junction
              }
         } else { 
#
# when the tool is not mounted, the tool tip is not the reference 
# junction any more. Then identify a new junction. Normaly this should be the
# junction that defines the pivot point.
#
              set ref_jct $sim_cfg_machine_tool_ref_junction
         } 

         SIM_set_current_ref_junction $ref_jct  
}

#=============================================================
proc VNC_coolant_on { } {
#=============================================================
  global  sound_files_library   
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  SIM_set_coolant "ON"
#  SIM_update
}


#=============================================================
proc VNC_coolant_off { } {
#=============================================================

  global  sound_files_library  
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  SIM_set_coolant "OFF"
#  SIM_update
}

#=============================================================
proc VNC_move_linear_mtcs { mode args } {
#=============================================================
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

# remove outer braces 
  set args [join $args]


  if { $mode != "CURRENT" } {
      VNC_set_feedrate_mode $mode
  }

  eval SIM_move_linear_mtcs $args

}

#=============================================================
proc VNC_move_linear_zcs { mode args } {
#=============================================================
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

# remove outer braces 
  set args [join $args]


  if { $mode != "CURRENT" } {
      VNC_set_feedrate_mode $mode
  }

  eval SIM_move_linear_zcs $args
}

#=============================================================
proc VNC_rapid_ab { } {
#=============================================================
  global sim_mtd_initialized mom_out_angle_pos
  global sim_4th_axis sim_5th_axis
  global xf yf zf B Cabs

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
#  SIM_move_linear_zcs    $sim_4th_axis $mom_out_angle_pos(0)  $sim_5th_axis $mom_out_angle_pos(1)
  SIM_move_linear_zcs    C $Cabs  B $B

}

#=============================================================
proc VNC_rapid_xyab { } {
#=============================================================
  global sim_mtd_initialized mom_pos mom_out_angle_pos
  global sim_4th_axis sim_5th_axis
  global xf yf zf B Cabs

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
  SIM_move_linear_zcs   X $xf Y $yf C $Cabs  B $B

#  if {[info exists sim_5th_axis] != 0 } {
#     SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1) $sim_4th_axis $mom_out_angle_pos(0)  $sim_5th_axis $mom_out_angle_pos(1)
#  } else {
#     if {[info exists sim_4th_axis] != 0 } {
#        SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1) $sim_4th_axis $mom_out_angle_pos(0)
#     } else {
#        SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1)
#     }
#  }
}

#=============================================================
proc VNC_rapid_xy { } {
#=============================================================
  global sim_mtd_initialized mom_pos 
  global xf yf

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
#  SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1) 
SIM_move_linear_zcs   X $xf Y $yf
}


#=============================================================
proc VNC_rapid_z { } {
#=============================================================
  global sim_mtd_initialized mom_pos 
  global zf

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
#  SIM_move_linear_zcs   Z $mom_pos(2) 
  SIM_move_linear_zcs   Z $zf
}

#=============================================================
proc VNC_rapid_xyz { } {
#=============================================================
  global sim_mtd_initialized mom_pos  mom_spindle_speed
  global xf yf zf

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]
#  VNC_coolant_on

#  SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1) Z $mom_pos(2)
SIM_move_linear_zcs   X $xf Y $yf Z $zf

}

#=============================================================
proc VNC_rapid_xyzab { } {
#=============================================================
  global sim_mtd_initialized mom_pos mom_out_angle_pos mom_spindle_speed
  global sim_4th_axis sim_5th_axis
  global xf yf zf B Cabs

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]
#  VNC_coolant_on
  SIM_move_linear_mtcs X $xf Y $yf Z $zf C $Cabs B $B


#  if {[info exists sim_5th_axis] != 0 } {
#     SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1) Z $mom_pos(2) $sim_4th_axis $mom_out_angle_pos(0)  $sim_5th_axis $mom_out_angle_pos(1)
#  } else {
#     if {[info exists sim_4th_axis] != 0 } {
#        SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1) Z $mom_pos(2) $sim_4th_axis $mom_out_angle_pos(0)
#     } else {
#        SIM_move_linear_zcs   X $mom_pos(0) Y $mom_pos(1) Z $mom_pos(2)
#     }
#  }
}

#=============================================================
proc VNC_linear_move { } {
#=============================================================
  global sim_mtd_initialized mom_spindle_speed
  global xf yf zf B Cabs

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "CUT"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]
#  VNC_coolant_on

  SIM_move_linear_zcs X $xf Y $yf Z $zf C $Cabs B $B

}
#=============================================================
proc VNC_linear_move_c { } {
#=============================================================
  global sim_mtd_initialized mom_spindle_speed
  global xf yf zf B Cabs

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "CUT"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]
#  VNC_coolant_on

  SIM_move_linear_zcs X $xfc Y $yfc

}

#=============================================================
proc VNC_fifth_axis_rotate_move { } {
#=============================================================
  global sim_mtd_initialized mom_out_angle_pos 
  global sim_5th_axis

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]
#  VNC_coolant_on

  SIM_move_linear_zcs  $sim_5th_axis $mom_out_angle_pos(1)
}

#=============================================================
proc VNC_fourth_axis_rotate_move { } {
#=============================================================
  global sim_mtd_initialized mom_out_angle_pos 
  global sim_4th_axis

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  VNC_set_feedrate_mode "RAPID"
  SIM_set_speed $mom_spindle_speed [VNC_ask_speed_mode]
#  VNC_coolant_on

  SIM_move_linear_zcs  $sim_4th_axis $mom_out_angle_pos(0)
}

#=============================================================
proc VNC_set_post_kinematics { zcs_jct X_axis Y_axis Z_axis {4th_axis ""} {5th_axis ""} {pivot_jct ""} {gauge_jct ""} } {
#=============================================================
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  SIM_set_post_kinematics $zcs_jct $X_axis $Y_axis $Z_axis  $4th_axis $5th_axis $pivot_jct $gauge_jct
}

#=============================================================
proc VNC_is_machine_tool_model_exists { } {
#=============================================================
  global sim_mtd_initialized

  # this function should realy check for whether a machine tool
  # model exist or not. for now, we check if simulation has 
  # been initialized properly
  
  return $sim_mtd_initialized  

}

#=============================================================
proc VNC_macro_def { macro_name } {
#=============================================================
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }


  SIM_macro_def $macro_name

}

#=============================================================
proc VNC_macro_end { macro_name } {
#=============================================================
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }


  SIM_macro_end $macro_name

}

#=============================================================
proc VNC_macro_call { macro_name } {
#=============================================================
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }


  SIM_macro_call $macro_name

}

#=============================================================
proc VNC_do_drill_cycle { } {
#=============================================================
  global sim_mtd_initialized 
  global mom_cycle_rapid_to_pos 
  global mom_cycle_feed_to_pos 
  global mom_cycle_retract_to_pos
  global mom_cycle_delay
  global mom_cycle_rapid_to 
  global mom_cycle_feed_to
  global mom_cycle_retract_to
  global mom_out_angle_pos
  global mom_cycle_spindle_axis
  global sim_4th_axis sim_5th_axis

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }
   
  set axis_name(0) X
  set axis_name(1) Y
  set axis_name(2) Z

  if {[info exists sim_5th_axis] != 0} {
    VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) \
       $sim_5th_axis $mom_out_angle_pos(1)
  } else {
    if {[info exists sim_4th_axis] != 0} {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) 
    } else {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) 
    }
  }

  VNC_move_linear_zcs CUT \
       X $mom_cycle_feed_to_pos(0) \
       Y $mom_cycle_feed_to_pos(1) \
       Z $mom_cycle_feed_to_pos(2) 

  VNC_move_linear_zcs RAPID \
       X $mom_cycle_retract_to_pos(0) \
       Y $mom_cycle_retract_to_pos(1) \
       Z $mom_cycle_retract_to_pos(2) 

#  VNC_move_linear_zcs RAPID \
#       $axis_name($mom_cycle_spindle_axis) $mom_cycle_rapid_to_pos($mom_cycle_spindle_axis)
}

#=============================================================
proc VNC_get_cycle_z { z_val } {
#=============================================================

  global sim_last_X sim_last_Y sim_last_Z
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return 0}

  set z [expr $sim_last_Z + $z_val]
  
  return $z
}


#=============================================================
proc VNC_set_cycle_feedrate { } {
#=============================================================
  global mom_cycle_feed_rate 
  global sim_mtd_initialized

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  SIM_set_cutting_mode "CUT"

  set sim_feedrate_mode "CUT"

  SIM_set_feed $mom_cycle_feed_rate [VNC_ask_feedrate_mode]
}

#=============================================================
proc VNC_do_drill_dwell_cycle { } {
#=============================================================
  global sim_mtd_initialized 
  global mom_cycle_rapid_to_pos 
  global mom_cycle_feed_to_pos 
  global mom_cycle_retract_to_pos
  global mom_cycle_delay
  global mom_out_angle_pos
  global sim_4th_axis sim_5th_axis
  global mom_cycle_rapid_to 
  global mom_cycle_feed_to
  global mom_cycle_retract_to
  global mom_cycle_spindle_axis

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  set axis_name(0) X
  set axis_name(1) Y
  set axis_name(2) Z

  if {[info exists sim_5th_axis] != 0} {
    VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) \
       $sim_5th_axis $mom_out_angle_pos(1)
  } else {
    if {[info exists sim_4th_axis] != 0} {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) 
    } else {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) 
    }
  }

  VNC_move_linear_zcs CUT \
       X $mom_cycle_feed_to_pos(0) \
       Y $mom_cycle_feed_to_pos(1) \
       Z $mom_cycle_feed_to_pos(2) 

  SIM_delay $mom_cycle_delay
  SIM_step

#  VNC_move_linear_zcs RAPID \
#       $axis_name($mom_cycle_spindle_axis) $mom_cycle_rapid_to_pos($mom_cycle_spindle_axis)

  VNC_move_linear_zcs RAPID \
       X $mom_cycle_retract_to_pos(0) \
       Y $mom_cycle_retract_to_pos(1) \
       Z $mom_cycle_retract_to_pos(2) 

}

#=============================================================
 proc vnc_update_last_tool_pos_and_axis { } {
#=============================================================
  global sim_last_X sim_last_Y sim_last_Z
  global sim_last_tool_axis_X sim_last_tool_axis_Y sim_last_tool_axis_Z

  # update last position and tool axis needed by s&v commands
  set pos_and_axis [SIM_ask_tool_position_zcs [SIM_ask_current_ref_junction] ]

  set sim_last_X [lindex $pos_and_axis 0] 
  set sim_last_Y [lindex $pos_and_axis 1]   
  set sim_last_Z [lindex $pos_and_axis 2] 

  set sim_last_tool_axis_X [lindex $pos_and_axis 3] 
  set sim_last_tool_axis_Y [lindex $pos_and_axis 4]   
  set sim_last_tool_axis_Z [lindex $pos_and_axis 5] 
}

#=============================================================
 proc vnc_get_tool_pos { dist } {
#=============================================================
  global sim_last_X sim_last_Y sim_last_Z
  global sim_last_tool_axis_X sim_last_tool_axis_Y sim_last_tool_axis_Z

  set x [expr  $sim_last_X + $dist * $sim_last_tool_axis_X]
  set y [expr  $sim_last_Y + $dist * $sim_last_tool_axis_Y]
  set z [expr  $sim_last_Z + $dist * $sim_last_tool_axis_Z]

  return "X $x Y $y Z $z"
}


#=============================================================
proc VNC_do_drill_deep_cycle { } {
#=============================================================
  global sim_mtd_initialized 
  global mom_cycle_rapid_to_pos 
  global mom_cycle_feed_to_pos 
  global mom_cycle_retract_to_pos
  global mom_cycle_rapid_to 
  global mom_cycle_feed_to
  global mom_cycle_retract_to
  global mom_cycle_step1
  global mom_out_angle_pos
  global sim_4th_axis sim_5th_axis
  global mom_cycle_spindle_axis

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  set axis_name(0) X
  set axis_name(1) Y
  set axis_name(2) Z

  if {[info exists sim_5th_axis] != 0} {
    VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) \
       $sim_5th_axis $mom_out_angle_pos(1)
  } else {
    if {[info exists sim_4th_axis] != 0} {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) 
    } else {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) 
    }
  }

  # update the last position 
  vnc_update_last_tool_pos_and_axis

  set abs_step1      [expr abs($mom_cycle_step1)]
  set abs_feed_to    [expr abs($mom_cycle_feed_to)]
  set abs_rapid_to   [expr abs($mom_cycle_rapid_to)]
  set abs_retract_to [expr abs($mom_cycle_retract_to)]

  if { $abs_step1 > 0 && $abs_feed_to > 0 } {

    set dist [expr $abs_step1 + $abs_rapid_to]
    set max_dist [expr $abs_step1 + $abs_feed_to]
    set prev_dist 0

    while { $dist < $max_dist } {

      VNC_move_linear_zcs CUT [ vnc_get_tool_pos [expr -1*$dist] ] 

      # for sinumeric controller the first step back is only 0.9 from the step
      if { $prev_dist > 0 } {
        VNC_move_linear_zcs RAPID [ vnc_get_tool_pos [expr -1*$prev_dist]] 
       } else {
        VNC_move_linear_zcs RAPID [ vnc_get_tool_pos [expr -0.90*$dist]] 
	   }


      set prev_dist $dist
      set dist [expr $dist + $abs_step1 ]
    }
  }

  VNC_move_linear_zcs CUT \
       X $mom_cycle_feed_to_pos(0) \
       Y $mom_cycle_feed_to_pos(1) \
       Z $mom_cycle_feed_to_pos(2) 

#  VNC_move_linear_zcs RAPID \
#       $axis_name($mom_cycle_spindle_axis) $mom_cycle_rapid_to_pos($mom_cycle_spindle_axis)

   VNC_move_linear_zcs RAPID \
       X $mom_cycle_retract_to_pos(0) \
       Y $mom_cycle_retract_to_pos(1) \
       Z $mom_cycle_retract_to_pos(2) 
}

#=============================================================
proc VNC_do_drill_break_chip_cycle { } {
#=============================================================

  VNC_do_drill_deep_cycle
}

#
# in case two steps are defined, one for down cutting and one
# for chip break use the following one.
#=============================================================
proc VNC_do_drill_break_chip_cycle_2_steps { } {
#=============================================================
  global sim_mtd_initialized 
  global mom_cycle_rapid_to_pos 
  global mom_cycle_feed_to_pos 
  global mom_cycle_retract_to_pos
  global mom_cycle_rapid_to 
  global mom_cycle_feed_to
  global mom_cycle_retract_to
  global mom_cycle_step1
  global mom_cycle_step2
  global mom_out_angle_pos
  global mom_cycle_spindle_axis
  global sim_4th_axis sim_5th_axis

  # check if simulation has been initialized properly
  if { !$sim_mtd_initialized } { return }

  set axis_name(0) X
  set axis_name(1) Y
  set axis_name(2) Z

  if {[info exists sim_5th_axis] != 0} {
    VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) \
       $sim_5th_axis $mom_out_angle_pos(1)
  } else {
    if {[info exists sim_4th_axis] != 0} {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) \
       $sim_4th_axis $mom_out_angle_pos(0) 
    } else {
      VNC_move_linear_zcs RAPID \
       X $mom_cycle_rapid_to_pos(0) \
       Y $mom_cycle_rapid_to_pos(1) \
       Z $mom_cycle_rapid_to_pos(2) 
    }
  }

  # update the last position 
  vnc_update_last_tool_pos_and_axis

  set abs_step1      [expr abs($mom_cycle_step1)]
  set abs_step2      [expr abs($mom_cycle_step2)]
  set abs_feed_to    [expr abs($mom_cycle_feed_to)]
  set abs_rapid_to   [expr abs($mom_cycle_rapid_to)]
  set abs_retract_to [expr abs($mom_cycle_retract_to)]

  if { $abs_step1 > 0 && $abs_feed_to > 0 } {

    set dist [expr $abs_step1 + $abs_rapid_to]

    set max_dist [expr $abs_step1 + $abs_feed_to]
    set prev_dist 0

    while { $dist < $max_dist } {

      VNC_move_linear_zcs CUT [ vnc_get_tool_pos [expr -1*$dist] ] 

	  set prev_dist [expr $dist - $abs_step2]
      VNC_move_linear_zcs RAPID [ vnc_get_tool_pos [expr -1*$prev_dist]] 

      set prev_dist $dist
      set dist [expr $dist + $abs_step1 ]
    }
  }

  VNC_move_linear_zcs CUT \
       X $mom_cycle_feed_to_pos(0) \
       Y $mom_cycle_feed_to_pos(1) \
       Z $mom_cycle_feed_to_pos(2) 

#  VNC_move_linear_zcs RAPID \
#       $axis_name($mom_cycle_spindle_axis) $mom_cycle_rapid_to_pos($mom_cycle_spindle_axis)

  VNC_move_linear_zcs RAPID \
       X $mom_cycle_retract_to_pos(0) \
       Y $mom_cycle_retract_to_pos(1) \
       Z $mom_cycle_retract_to_pos(2) 
}

#=============================================================
proc VNC_do_tap_cycle { } {
#=============================================================

  VNC_do_drill_cycle
}

#=============================================================
proc VNC_do_bore_cycle { } {
#=============================================================

  VNC_do_drill_cycle
}

#=============================================================
proc VNC_do_bore_drag_cycle { } {
#=============================================================

  VNC_do_drill_cycle
}

#=============================================================
proc VNC_do_bore_no_drag_cycle { } {
#=============================================================

  VNC_do_drill_cycle
}

#=============================================================
proc VNC_do_bore_manual_cycle { } {
#=============================================================

  VNC_do_drill_cycle
}

#=============================================================
proc VNC_do_bore_dwell_cycle { } {
#=============================================================

  VNC_do_drill_dwell_cycle
}

#=============================================================
proc VNC_do_bore_back_cycle { } {
#=============================================================

  VNC_do_drill_cycle
}

#=============================================================
proc VNC_do_bore_manual_dwell_cycle { } {
#=============================================================

  VNC_do_drill_dwell_cycle
}

#=============================================================
proc VNC_nurbs_move {  } {
#=============================================================
	global sim_mtd_initialized
	global mom_nurbs_knot_count
	global mom_nurbs_knots
	global mom_nurbs_point_count
	global mom_nurbs_points
	global mom_nurbs_order

	# check if simulation has been initialized properly
	if { !$sim_mtd_initialized } { return }

  	set knots [list]
	set points [list]
	
  	for {set i 0} {$i < $mom_nurbs_knot_count} {incr i} \
		{  lappend knots $mom_nurbs_knots($i) }
 
	for {set i 0} {$i < $mom_nurbs_point_count} {incr i} \
	{
		lappend points $mom_nurbs_points($i,0)
		lappend points $mom_nurbs_points($i,1)
		lappend points $mom_nurbs_points($i,2)
	}

	SIM_move_nurbs_zcs	$mom_nurbs_point_count $mom_nurbs_order \
								$mom_nurbs_knot_count $knots $points
}


#----------------------------------------------------------------------------
#
# Function VNC_reload_kinematics updates the post kinematics variables according to 
# the machine tool kinematics model. This function also reload advance kinematics DLL
# data is requested.
#
#----------------------------------------------------------------------------
proc VNC_reload_kinematics {} {

  global sim_prev_zcs sim_4th_axis  sim_5th_axis 
  global sim_cfg_setup_post_kinematics_sw
  global sim_mtd_initialized
  global sim_cfg_setup_post_kinematics_ref_junction
  global sim_cfg_setup_advance_kinematics_sw

  if { !$sim_mtd_initialized || $sim_cfg_setup_post_kinematics_sw == 0} { return }


  # define the reference junction for defining the 4th and 5th
  # axis post kinematics data. It can be the ZCS junction or 
  # an explicit one. Offsets and vectors for kinematics are defined
  # i.r.t this junction. If you use ZCS, make sure that the right 
  # ZCS is active when this procedure is called.
  if { $sim_cfg_setup_post_kinematics_ref_junction == "_SIM_CURRENT_ZCS_JUNCTION" } {
     set current_zcs [SIM_ask_current_zcs_junction]
  } else {
     set current_zcs $sim_cfg_setup_post_kinematics_ref_junction
  }

  #
  # set kinematics based on the machine tool kinematics model
  # current_zcs represents the MCS used with this machine
  if { [ VNC_is_machine_tool_model_exists ] } {
      if { $sim_prev_zcs != $current_zcs } {
        if {[info exists sim_5th_axis] != 0} {
         VNC_set_post_kinematics $current_zcs X Y Z $sim_4th_axis $sim_5th_axis
        } else {
          if {[info exists sim_4th_axis] != 0} {
             VNC_set_post_kinematics $current_zcs X Y Z $sim_4th_axis 
          } else { 
             VNC_set_post_kinematics $current_zcs X Y Z
          }
        } 
         set sim_prev_zcs $current_zcs
      }
  }

# reload kinematics in the UG kinematics data structures...
  MOM_reload_kinematics

# 
# if asked - reload kinematics data in advance kinematics DLL
#
  if { $sim_cfg_setup_advance_kinematics_sw == 1} {
    vnc_reload_advance_kinematics
  }
}

#
# reload kinematics variables of the UG advance kinematics 
# data.
#
proc vnc_reload_advance_kinematics {} {

  global sim_advance_kin_dll_loaded
  global sim_4th_axis
  global sim_5th_axis

# The globals below are to enable ufunc to read the kinematics
# value of center_offset_x/y/z is the same with  above 
# mom_kin_4th_axis_center_offset(i), tcl/ufun can not read array

#
# TCL ufun cannot read array, so convert to variables
#

  global mom_kin_4th_axis_center_offset
  global mom_kin_4th_axis_center_offset_x
  global mom_kin_4th_axis_center_offset_y
  global mom_kin_4th_axis_center_offset_z

  global mom_kin_4th_axis_angles
  global mom_kin_4th_axis_angles_a1
  global mom_kin_4th_axis_angles_a2

  global mom_kin_4th_axis_vector
  global mom_kin_4th_axis_vector_i
  global mom_kin_4th_axis_vector_j
  global mom_kin_4th_axis_vector_k

  global mom_kin_5th_axis_center_offset
  global mom_kin_5th_axis_center_offset_x
  global mom_kin_5th_axis_center_offset_y
  global mom_kin_5th_axis_center_offset_z

  global mom_kin_5th_axis_angles
  global mom_kin_5th_axis_angles_a1
  global mom_kin_5th_axis_angles_a2

  global mom_kin_5th_axis_vector
  global mom_kin_5th_axis_vector_i
  global mom_kin_5th_axis_vector_j
  global mom_kin_5th_axis_vector_k

#  global mom_kin_4th_to_tracking_point
#  global mom_kin_4th_to_tracking_point_x
#  global mom_kin_4th_to_tracking_point_y
#  global mom_kin_4th_to_tracking_point_z

#  global mom_kin_5th_to_tracking_point
#  global mom_kin_5th_to_tracking_point_x
#  global mom_kin_5th_to_tracking_point_y
#  global mom_kin_5th_to_tracking_point_z

  global mom_kin_gauge_to_pivot
  global mom_kin_gauge_to_pivot_x
  global mom_kin_gauge_to_pivot_y
  global mom_kin_gauge_to_pivot_z

  global mom_kin_4th_axis_limit_action
  global mom_kin_5th_axis_limit_action

  global mom_kin_4th_axis_plane
  global mom_kin_5th_axis_plane

  #
  #  ######### 4th axis 
  #

if {[info exists sim_4th_axis] != 0} {
  set mom_kin_4th_axis_limit_action             "Retract / Re-Engage"
  
  set mom_kin_4th_axis_center_offset_x $mom_kin_4th_axis_center_offset(0)
  set mom_kin_4th_axis_center_offset_y $mom_kin_4th_axis_center_offset(1) 
  set mom_kin_4th_axis_center_offset_z $mom_kin_4th_axis_center_offset(2)

#  set mom_kin_gauge_to_pivot_x $mom_kin_gauge_to_pivot(0)
#  set mom_kin_gauge_to_pivot_y $mom_kin_gauge_to_pivot(1)
#  set mom_kin_gauge_to_pivot_z $mom_kin_gauge_to_pivot(2)

#  set mom_kin_4th_to_tracking_point_x $mom_kin_4th_to_tracking_point(0)
#  set mom_kin_4th_to_tracking_point_y $mom_kin_4th_to_tracking_point(1)
#  set mom_kin_4th_to_tracking_point_z $mom_kin_4th_to_tracking_point(2)

  switch $mom_kin_4th_axis_plane {
    "XY" -
    "YX"
         {
           set mom_kin_4th_axis_vector_i                  0.0
           set mom_kin_4th_axis_vector_j                  0.0
           set mom_kin_4th_axis_vector_k                  1.0
         }
    "YZ" -
    "ZY"
         {
           set mom_kin_4th_axis_vector_i                  1.0
           set mom_kin_4th_axis_vector_j                  0.0
           set mom_kin_4th_axis_vector_k                  0.0
         }
    "ZX" -
    "XZ"
         {
           set mom_kin_4th_axis_vector_i                  0.0
           set mom_kin_4th_axis_vector_j                  1.0
           set mom_kin_4th_axis_vector_k                  0.0
         }
  default
         {
           if {[info exists mom_kin_4th_axis_vector]} {
               set mom_kin_4th_axis_vector_i $mom_kin_4th_axis_vector(0)
               set mom_kin_4th_axis_vector_j $mom_kin_4th_axis_vector(1)
               set mom_kin_4th_axis_vector_k $mom_kin_4th_axis_vector(2)
           } elseif {[info exists mom_kin_4th_axis_angles]} {
               set mom_kin_4th_axis_angles_a1 $mom_kin_4th_axis_angles(0)
               set mom_kin_4th_axis_angles_a2 $mom_kin_4th_axis_angles(1)
           } else {
               set mom_kin_4th_axis_vector_i                  0.0
               set mom_kin_4th_axis_vector_j                  0.0
               set mom_kin_4th_axis_vector_k                  0.0
               set mom_kin_4th_axis_angles_a1                 0.0
               set mom_kin_4th_axis_angles_a2                 0.0
               MOM_SIM_ERROR_MSG "ERROR: Machine's 4th axis Kinematics is INCONSISTANT"
	           return TCL_ERROR
           }
         }
  }
}

  #
  #  ######### 5th axis 
  #
if {[info exists sim_5th_axis] != 0} {
  set mom_kin_5th_axis_limit_action             "Retract / Re-Engage"

  set mom_kin_5th_axis_center_offset_x $mom_kin_5th_axis_center_offset(0)
  set mom_kin_5th_axis_center_offset_y $mom_kin_5th_axis_center_offset(1) 
  set mom_kin_5th_axis_center_offset_z $mom_kin_5th_axis_center_offset(2)

  switch $mom_kin_5th_axis_plane {
    "XY" -
    "YX"
         {
           set mom_kin_5th_axis_vector_i                  0.0
           set mom_kin_5th_axis_vector_j                  0.0
           set mom_kin_5th_axis_vector_k                  1.0
         }
    "YZ" -
    "ZY"
         {
           set mom_kin_5th_axis_vector_i                  1.0
           set mom_kin_5th_axis_vector_j                  0.0
           set mom_kin_5th_axis_vector_k                  0.0
         }
    "ZX" -
    "XZ"
         {
           set mom_kin_5th_axis_vector_i                  0.0
           set mom_kin_5th_axis_vector_j                  1.0
           set mom_kin_5th_axis_vector_k                  0.0
         }
  default
         {
           if {[info exists mom_kin_5th_axis_vector]} {
               set mom_kin_5th_axis_vector_i $mom_kin_5th_axis_vector(0)
               set mom_kin_5th_axis_vector_j $mom_kin_5th_axis_vector(1)
               set mom_kin_5th_axis_vector_k $mom_kin_5th_axis_vector(2)
           } elseif {[info exists mom_kin_5th_axis_angles]} {
               set mom_kin_5th_axis_angles_a1 $mom_kin_5th_axis_angles(0)
               set mom_kin_5th_axis_angles_a2 $mom_kin_5th_axis_angles(1)
           } else {
               set mom_kin_5th_axis_vector_i                  0.0
               set mom_kin_5th_axis_vector_j                  0.0
               set mom_kin_5th_axis_vector_k                  0.0
               set mom_kin_5th_axis_angles_a1                 0.0
               set mom_kin_5th_axis_angles_a2                 0.0
               MOM_SIM_ERROR_MSG "ERROR: Machine's 5th axis Kinematics is INCONSISTANT"
	           return TCL_ERROR
           }
         }
  }
}

  set uf_library "ugpadvkins.dll"

  # first load advance kinematics DLL, then run command from that DLL.
  if { $sim_advance_kin_dll_loaded == 0 } {
     MOM_run_user_function [MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]$uf_library ufusr
     set sim_advance_kin_dll_loaded 1
  }

# now reload kinematics in the advanve kinematics DLL
  MOM_load_kinematics

}

proc vnc_init_config { } {

  global sim_cfg_kin_machine_type
  global sim_cfg_4th_axis_name
  global sim_cfg_4th_axis_name
  global sim_cfg_5th_axis_name
  global sim_cfg_driver_name
  global sim_cfg_linear_axes_map 
  global sim_cfg_rotary_axes_map 
  global sim_cfg_linear_axes_config  
  global sim_cfg_rotary_axes_config 
  global sim_cfg_4th_axis_rotary_dir_mode 
  global sim_cfg_5th_axis_rotary_dir_mode 
  global sim_cfg_4th_axis_allow_reversal
  global sim_cfg_5th_axis_allow_reversal 
  global sim_cfg_debug
  global sim_cfg_tool_mounting_component
  global sim_cfg_tool_mounting_junction
  global sim_cfg_tool_mounting_time
  global sim_cfg_tool_unmounting_time
  global sim_cfg_machine_tool_ref_junction
  global sim_cfg_setup_post_kinematics_sw
  global sim_cfg_setup_post_kinematics_ref_junction 
  global sim_cfg_duration_callback_fct
  global sim_cfg_tool_change_motion_callback_fct
  global sim_cfg_tool_ref_junction
  global sim_cfg_mounting_zcs_component
  global sim_cfg_setup_advance_kinematics_sw

  global mom_kin_tool_change_time

  global sim_4th_axis
  global sim_5th_axis
  global sim_num_axis

#
#  the logical names of the 4'th and 5'th axis. 
#
 set sim_num_axis 3
 if {[info exists sim_cfg_4th_axis_name] != 0 } {
    set sim_4th_axis $sim_cfg_4th_axis_name
    set sim_num_axis 4
 }

 if {[info exists sim_cfg_5th_axis_name] != 0 } {
    set sim_5th_axis $sim_cfg_5th_axis_name
    set sim_num_axis 5
 }

#
# name of driver (optional)
#
 if {[info exists sim_cfg_driver_name] == 0} {
   set sim_cfg_driver_name "Generic Driver"
 }

#
# define mapping between logical and phisical axes
#
 if {[info exists sim_cfg_linear_axes_map] == 0} {
     MOM_SIM_ERROR_MSG "Linear axis mapping not defined. Please set sim_cfg_linear_axes_map."
	 return TCL_ERROR
 } else {
     SIM_set_linear_axes_map $sim_cfg_linear_axes_map
 }

 if {[info exists sim_cfg_rotary_axes_map] != 0} {
     SIM_set_rotary_axes_map $sim_cfg_rotary_axes_map
 } else {
    if { $sim_num_axis > 3 } {
     MOM_SIM_ERROR_MSG "Rotary axis mapping not defined. Please set sim_cfg_rotary_axes_map."
	 return TCL_ERROR
    }
 }

#
# define the linear and rotary axes that participate in execution of the
# motion. Use the logical names of the axes.
# For example, for this machine all axes participate in executing a 
# certain motion. so all axes are included in axes configuration. 
#
 if {[info exists sim_cfg_linear_axes_config] == 0} {
     MOM_SIM_ERROR_MSG "Linear axis configuration not defined. Please set sim_cfg_linear_axes_config."
	 return TCL_ERROR
 } else {
     SIM_set_linear_axes_config $sim_cfg_linear_axes_config
 }

 if {[info exists sim_cfg_rotary_axes_config] != 0} {
     SIM_set_rotary_axes_config $sim_cfg_rotary_axes_config
 } else {
    if { $sim_num_axis > 3 } {
     MOM_SIM_ERROR_MSG "Rotary axis configuration not defined. Please set sim_cfg_rotary_axes_config."
	 return TCL_ERROR
    }
 }
 
#
# specify on which K-component to mount the ZCS junction. This
# is the junction that matchs the MSYS coordinate system. The 
# NC program coordinates are i.r.t this coordinate system
#
 if {[info exists sim_cfg_mounting_zcs_component] == 0} {
     MOM_SIM_ERROR_MSG "Mounting K-Component for ZCS is not specified. Please set sim_cfg_mounting_zcs_component."
	 return TCL_ERROR
 }

# 
# set rotation direction mode which specify in which direction to rotate
# the rotary axis when moving to target position. by default shortest 
# path is applied (if not specified)
#
 if {[info exists sim_cfg_4th_axis_rotary_dir_mode] != 0} {
     SIM_set_axis_rotary_dir_mode $sim_4th_axis $sim_cfg_4th_axis_rotary_dir_mode
 }
   
 if {[info exists sim_cfg_5th_axis_rotary_dir_mode] != 0} {
     SIM_set_axis_rotary_dir_mode $sim_5th_axis $sim_cfg_5th_axis_rotary_dir_mode
 }
   
#
# set whether axis reversal is allowed. i.e. when axis cannot reach target
# position through the shortest path whether going on the longest path is
# allowed. For example if current position is 350 and target is 10 is limits
# don't allow going to 10. this controls whether it can rotate 240 degrees to 
# 10. By default it is allowed for all rotary axes.
#
 if {[info exists sim_cfg_4th_axis_allow_reversal] != 0 && $sim_num_axis > 3 } {
     SIM_set_axis_allow_reversal $sim_4th_axis $sim_cfg_4th_axis_allow_reversal
 }

 if {[info exists sim_cfg_5th_axis_allow_reversal] != 0 && $sim_num_axis > 4 } {
     SIM_set_axis_allow_reversal $sim_5th_axis $sim_cfg_5th_axis_allow_reversal
 }

#
# set the callback TCL procedure for computing the duration time. If not
# set a default one is provided.
# The function has the following arguments
#   CalculateDurationTime { linear_or_angular delta } where
#      linear_or_angular is either "LINEAR" or "ANGULAR" which defines
#      whether the incremental displacement (delta) is linear(in current units) 
#      or angular(in degrees).
#
#   The function must return the duration in seconds to perform this delta
#   angular or linear motion.
#
 if {[info exists sim_cfg_duration_callback_fct] != 0 } {
     SIM_set_duration_callback_fct $sim_cfg_duration_callback_fct
 } else {
     SIM_set_duration_callback_fct "CalculateDurationTime"
 }

#
# set debug options  
# Set what outputs we want to see. Valid tokens are:
# SIM_DBG_OUTPUT_ERROR, SIM_DBG_OUTPUT_NC_CMD, SIM_DBG_OUTPUT_CEVENT, 
# SIM_DBG_OUTPUT_INPUT_SV_EVENT, SIM_DBG_OUTPUT_OUTPUT_SV_EVENT, 
# SIM_DBG_OUTPUT_PROCESSED_SV_EVENT, SIM_DBG_OUTPUT_SV_CMD, SIM_DBG_OUTPUT_ALL
#
# if {[info exists sim_cfg_debug] != 0} {
#    SIM_dbg_start "============== Start Debugging for $sim_cfg_driver_name ========"
#    set dbg_cmd "SIM_dbg_set_output ON $sim_cfg_debug"
#    eval $dbg_cmd
#  }

#
# define on which K-component the tool is mounted and to which junction
# it is attached.
#
 if {[info exists sim_cfg_tool_mounting_component] == 0} {
     MOM_SIM_ERROR_MSG "K-Component where the tool is mounted is not set. Please set sim_cfg_tool_mounting_component."
	 return TCL_ERROR
 } 

 if {[info exists sim_cfg_tool_mounting_junction] == 0} {
     MOM_SIM_ERROR_MSG "Tool mounting junction is not set. Please set sim_cfg_tool_mounting_junction."
	 return TCL_ERROR
 } 

#
# the time it takes to mount and unmount a tool in seconds. if not define mount and unmount
# are set as half of the mom_kin_tool_change_time value if defined.
#
  if {[info exists sim_cfg_tool_mounting_time] == 0} {
   if {[info exists mom_kin_tool_change_time] != 0} {
     set sim_cfg_tool_mounting_time [expr 0.5*$mom_kin_tool_change_time]
   } else {
     set sim_cfg_tool_mounting_time 0.0
   }
  }

  if {[info exists sim_cfg_tool_unmounting_time] == 0} {
   if {[info exists mom_kin_tool_change_time] != 0} {
     set sim_cfg_tool_unmounting_time [expr 0.5*$mom_kin_tool_change_time]
   } else {
     set sim_cfg_tool_unmounting_time 0.0
   }
  }

# When the machine is instructed to move, by default (if variable not set) this means
# to bring the tool tip position to target position. Any other junction can be 
# defined as the reference junction. Make sure the junction is attached to 
# the right K-component.
# 
# variable sim_cfg_tool_ref_junction 
#

# when the tool is not mounted on the machine a new reference
# junction has to be defined to replace the tool tip junction. 
# define here the new junction
 if {[info exists sim_cfg_machine_tool_ref_junction] == 0} {
     MOM_SIM_ERROR_MSG "Machine tool reference junction not defined. Please set sim_cfg_machine_tool_ref_junction."
	 return TCL_ERROR
 }

#
# The following switch control whether to setup the post kinematics
# based on the machine configuration. If not set, the pos kinematics 
# of the machine has to be defined outside VNC.
# The kinematics parameters that are set are defined in sim_mtd_kinematics.tcl.
# Those parameters are just the kinematics geometry like axes limits, rotation
# centers and others 
 if {[info exists sim_cfg_setup_post_kinematics_sw] == 0} {
     set sim_cfg_setup_post_kinematics_sw 0
 }
  
#
# if sim_cfg_setup_post_kinematics_sw is set then...
# define the reference junction for defining the 4th and 5th
# axis post kinematics data. It can be the current ZCS junction or 
# an explicit one. Offsets and vectors for POST kinematics are defined
# i.r.t this junction. If you use ZCS, make sure that the right 
# ZCS is active when this procedure is called.
# To use the current ZCS set the following to "_SIM_CURRENT_ZCS_JUNCTION"
# otherwise use an explicit junction.
 if {$sim_cfg_setup_post_kinematics_sw != 0 && [info exists sim_cfg_setup_post_kinematics_ref_junction] == 0} {
     MOM_SIM_ERROR_MSG "Reference junction for post kinematics not defined. Please set sim_cfg_setup_post_kinematics_ref_junction."
	 return TCL_ERROR
 }
}

#
# Also, set whether to reload the advance kinematics variables 
# in case the advance kinematics DLL is used
 if {[info exists sim_cfg_setup_advance_kinematics_sw] == 0} {
     set sim_cfg_setup_advance_kinematics_sw 0
 }
  
#
#----- This is a sample VNC configuration. Copy this procedure 
#      to your machine driver and set variables according to your
#      machine tool configuration and controller behavior
#      rename this function to VNC_configure_driver
#
proc VNC_configure_driver_sample { } {

  global sim_cfg_kin_machine_type
  global sim_cfg_4th_axis_name
  global sim_cfg_4th_axis_name
  global sim_cfg_5th_axis_name
  global sim_cfg_driver_name
  global sim_cfg_linear_axes_map 
  global sim_cfg_rotary_axes_map 
  global sim_cfg_linear_axes_config  
  global sim_cfg_rotary_axes_config 
  global sim_cfg_4th_axis_rotary_dir_mode 
  global sim_cfg_5th_axis_rotary_dir_mode 
  global sim_cfg_4th_axis_allow_reversal
  global sim_cfg_5th_axis_allow_reversal 
  global sim_cfg_debug
  global sim_cfg_tool_mounting_component
  global sim_cfg_tool_mounting_junction
  global sim_cfg_tool_mounting_time
  global sim_cfg_tool_unmounting_time
  global sim_cfg_machine_tool_ref_junction
  global sim_cfg_setup_post_kinematics_sw
  global sim_cfg_setup_post_kinematics_ref_junction 
  global sim_cfg_duration_callback_fct
  global sim_cfg_tool_change_motion_callback_fct
  global sim_cfg_tool_ref_junction
  global sim_cfg_end_of_prog_callback_fct
  global sim_cfg_mounting_zcs_component
  global sim_cfg_setup_advance_kinematics_sw

# (REQUIRED)
#  the logical names of the 4'th and 5'th axis. 
#  must be set for 4 or 5 axis machine
#
#  set sim_cfg_4th_axis_name A
#  set sim_cfg_5th_axis_name C

#(REQUIRED)
# define mapping between logical and phisical axes as they named
# on the machine. This has to be defined for the linear and rotary axes.
#  set sim_cfg_linear_axes_map [concat X X Y Y Z Z]
#  set sim_cfg_rotary_axes_map [concat A A C C]

# (REQUIRED)
# define the linear and rotary axes that participate in execution of the
# motion. Use the logical names of the axes.
# For example, for this machine all axes participate in executing a 
# certain motion. so all axes are included in axes configuration. 
#  set sim_cfg_linear_axes_config [concat X Y Z] 
#  set sim_cfg_rotary_axes_config [concat A C]

# (REQUIRED)
# specify on which K-component to mount the ZCS junction. This
# is the junction that matchs the MSYS coordinate system. The 
# NC program coordinates are i.r.t this coordinate system
# Note: don't position ZCS on a K-component that moves or rotate 
# since ZCS most oftime has to be stationary. Except for example 
# a table mounted on linear axis where the control expect the 
# coordinates in the moving ZCS coordinate system. In that case
# the ZCS is mounted on the moving linear axis
# set sim_cfg_mounting_zcs_component "MACHINE_BASE"

#(REQUIRED)
# define on which K-component the tool is mounted and to which junction
# it is attached.
#  set sim_cfg_tool_mounting_component "SPINDLE"
#  set sim_cfg_tool_mounting_junction  "SPINDLE@TOOL_MOUNT_JCT"

#(REQUIRED)
# when the tool is not mounted on the machine a new reference
# junction has to be defined to replace the tool tip junction. 
# this reference junction is used to move the machine without a tool.
# set sim_cfg_machine_tool_ref_junction "SPINDLE@TOOL_MOUNT_JCT"

# (OPTIONAL)
# set rotation direction mode which specify in which direction to rotate
# the rotary axis when moving to target position. by default shortest 
# path is applied (if not specified)
#  set sim_cfg_4th_axis_rotary_dir_mode "ALWAYS_SHORTEST"
#  set sim_cfg_5th_axis_rotary_dir_mode "ALWAYS_SHORTEST"

# (OPTIONAL)
# set whether axis reversal is allowed. i.e. when axis cannot reach target
# position through the shortest path whether going on the longest path is
# allowed. For example if current position is 350 and target is 10 is limits
# don't allow going to 10. this controls whether it can rotate 240 degrees to 
# 10. By default it is allowed for all rotary axes.
#  set sim_cfg_4th_axis_allow_reversal "YES"
#  set sim_cfg_5th_axis_allow_reversal "YES"

# (OPTIONAL)
# set the callback TCL procedure for computing the duration time. If not
# set a default one is provided.
# The function has the following arguments
#   CalculateDurationTime { linear_or_angular delta } where
#      linear_or_angular is either "LINEAR" or "ANGULAR" which defines
#      whether the incremental displacement (delta) is linear(in current units) 
#      or angular(in degrees).
#
#   The function must return the duration in seconds to perform this delta
#   angular or linear motion.
#
# set sim_cfg_duration_callback_fct "SIM_calculate_duration_time_CB"

# (OPTIONAL)
# name of driver
#  set sim_cfg_driver_name "5-Axis Generic Driver" 

# (OPTIONAL)
# This defines the function that is called to perform the motion to the 
# tool change position. Note that this is just the motion not the actual tool change.
# if not set, tool change is done in current position. 
# The function gets no arguments and return nothing. It is just performs the
# motion.
#
#  set sim_cfg_tool_change_motion_callback_fct "SIM_tool_change_path_CB"

# (OPTIONAL)
# This function is called when end of program is triggered. 
# The function gets no arguments and returns nothing. 
# if not defined. no motion is performed.
#
#  set sim_cfg_end_of_prog_callback_fct "SIM_end_of_program_CB"

# (OPTIONAL)
# set debug options  
# Set what outputs we want to see. Valid tokens are:
# SIM_DBG_OUTPUT_ERROR, SIM_DBG_OUTPUT_NC_CMD, SIM_DBG_OUTPUT_CEVENT, 
# SIM_DBG_OUTPUT_INPUT_SV_EVENT, SIM_DBG_OUTPUT_OUTPUT_SV_EVENT, 
# SIM_DBG_OUTPUT_PROCESSED_SV_EVENT, SIM_DBG_OUTPUT_SV_CMD, SIM_DBG_OUTPUT_ALL
# 
#  set sim_cfg_debug "SIM_DBG_OUTPUT_INPUT_SV_EVENT SIM_DBG_OUTPUT_CEVENT"
#
# when debug is set you can use the following command to write text
# SIM_dbg_write_message ">>> This is a testing line <<<<"
#

#(OPTIONAL)
# the time it takes to mount and unmount a tool in seconds. if not define mount and unmount
# are set as half of the mom_kin_tool_change_time value if defined.
# set sim_cfg_tool_mounting_time   6
# set sim_cfg_tool_unmounting_time 6
 
# (OPTIONAL)
# When the machine is instructed to move, by default (if variable not set) this means
# to bring the tool tip position to target position. Any other junction can be 
# defined as the reference junction. Make sure the junction is attached to 
# the right K-component.
# set sim_cfg_tool_ref_junction "SPINDLE@TOOL_MOUNT_JCT"
     
#(OPTIONAL)
# The following switch control whether to setup the post kinematics
# based on the machine configuration. If not set, the pos kinematics 
# of the machine has to be defined outside VNC.
# The kinematics parameters that are set are defined in sim_mtd_kinematics.tcl.
# Those parameters are just the kinematics geometry like axes limits, rotation
# centers and others 
# Variable values:0 or 1
# set sim_cfg_setup_post_kinematics_sw 1

#(OPTIONAL)
# if sim_cfg_setup_post_kinematics_sw is set then...
# define the reference junction for defining the 4th and 5th
# axis post kinematics data. It can be the current ZCS junction or 
# an explicit one. Offsets and vectors for POST kinematics are defined
# i.r.t this junction. If you use ZCS, make sure that the right 
# ZCS is active when this procedure is called.
# To use the current ZCS set the following to "_SIM_CURRENT_ZCS_JUNCTION"
# otherwise use an explicit junction.
#  set sim_cfg_setup_post_kinematics_ref_junction "ROT_JCT"

# (OPTIONAL)
# specify whether to setup the advance kinematics DLL too. this will
# load the advance kinematics DLL and reload its kinematics
# by default it is not loaded (set to 0)
# Variale values: 0 or 1
# set sim_cfg_setup_advance_kinematics_sw 1

#
# use the following commands to customize the simulation dialog. See Custom
# Dialog Spec for command parameters.
#

# SIM_dialog_start "Generic Driver Controller"

# system items
# SIM_dialog_add_item    "SYS_ACTIVE_TOOL_ITEM     S1"
# SIM_dialog_add_item    "SYS_SPEED_ITEM           S2" 
# SIM_dialog_add_item    "SYS_FEED_ITEM            S3 ATTACHMENT=<LEFT 0 82>"
# SIM_dialog_add_item    "SYS_MACHINE_TIME_ITEM    S4"
# SIM_dialog_add_item    "SYS_COOLANT_ITEM         S5 ATTACHMENT=<LEFT 0 30>"
# SIM_dialog_add_item    "SYS_POSITION_ITEM        S6"
# SIM_dialog_add_item    "SYS_PROGRAM_WINDOW_ITEM  S7"
# SIM_dialog_add_item    "SYS_MESSAGE_WINDOW_ITEM  S8"
# SIM_dialog_add_item    "SYS_SHOW_IPW_ITEM        S9"
# SIM_dialog_add_item    "SYS_STEP_CONTROL_ITEM    S10"
# SIM_dialog_add_item    "SYS_ANIMATION_SPEED_ITEM S11"

# user items
# SIM_dialog_add_item     "USR_SEPARATOR_ITEM      U1"
# SIM_dialog_add_item     "USR_LABEL_ITEM          U2 LABEL=<my label>"
# SIM_dialog_add_item     "USR_STRING_ITEM         U3 LABEL=<my string> VALUE=<string value>"
# SIM_dialog_add_item     "USR_RADIO_ITEM          U4 LABEL=<my radio> ORIENTATION=<HORIZONTAL> SPAN=<3> LIST=<radio1,radio2,radio3> VALUE=<1> FORMAT=<CHECK_BOX>"

# SIM_dialog_end 

#
# Customized items can be changed while running machine tool driver. Those
# commands can be executed after MTD has been initialized
#
# For example for the position item to show just the linear axis X, Y and Z
#    SIM_dialog_set_item "S6 LIST=<X,Y,Z>"
# 
# or for example to dim the collant item do
#    SIM_dialog_set_item "S5 SENSITIVE=<OFF>" 
#
# For user item all attributes can change
#   SIM_dialog_set_item     "U2 LABEL=<new label> SENSITIVE=<OFF>"
#   SIM_dialog_set_item     "U3 LABEL=<new label> VALUE=<new value> SENSITIVE=<OFF>"
#   SIM_dialog_set_item     "U4 LABEL=<new label> VALUE=<2> SENSITIVE=<OFF>"
#
}

